<?php 

$id = filter_input(INPUT_GET, 'id', FILTER_SANITIZE_STRING);

$configuration = Pronamic_WordPress_IDeal_ConfigurationsRepository::getConfigurationById($id);

?>
<div class="wrap">
	<?php screen_icon(Pronamic_WordPress_IDeal_Plugin::SLUG); ?>

	<h2>
		<?php _e('iDEAL Tests', 'pronamic_ideal'); ?>
	</h2>

	<?php if($configuration == null): ?>

	<p>
		<?php printf(__('We could not find any feed with the ID "%s".', 'pronamic_ideal'), $id); ?>
	</p>

	<?php else: ?>

	<?php $testsLink = admin_url(Pronamic_WordPress_IDeal_Admin::getConfigurationTestsLink($configuration->getId())); ?>

	<div>
		<h3>
			<?php _e('Info', 'pronamic_ideal'); ?>
		</h3>

		<table class="form-table">
			<tr>
				<th scope="row">
					<?php _e('ID', 'pronamic_ideal'); ?>
				</th>
				<td>
					<?php echo $configuration->getId(); ?>
				</td>
			</tr>
			<tr>
				<th scope="row">
					<?php _e('Name', 'pronamic_ideal'); ?>
				</th>
				<td>
					<?php echo $configuration->getName(); ?>
				</td>
			</tr>
		</table>
	</div>

	<?php if($configuration->getVariant() instanceof Pronamic_IDeal_VariantAdvanced): ?>

	<div>
		<h3>
			<?php _e('Retrieve Issuers Lists', 'pronamic_ideal'); ?>
		</h3>

		<?php 

		$lists = Pronamic_WordPress_IDeal_IDeal::getIssuersLists($configuration);

		if($lists): ?>

		<ul>
			<?php foreach($lists as $name => $list): ?>
			<li>
				<strong><?php echo $name; ?></strong>

				<?php if($list): ?>
				<ul>
					<?php foreach($list as $issuer): ?>
					<li>
						<?php echo $issuer->getName(); ?>
					</li>
					<?php endforeach; ?>
				</ul>
				<?php endif; ?>
			</li>
			<?php endforeach; ?>
		</ul>
		
		<h3>
			<?php _e('Mandatory Tests', 'pronamic_ideal'); ?>
		</h3>

		<form method="post" action="" target="_blank">
			<?php 
			
			wp_nonce_field('test', 'pronamic_ideal_nonce');

			echo Pronamic_IDeal_HTML_Helper::issuersSelect('pronamic_ideal_issuer_id', $lists);
			
			foreach(array(1, 2, 3, 4, 5, 7) as $testCase) {
				$name = sprintf(__('Test Case %s', 'pronamic_ideal'), $testCase);
				
				submit_button($name, 'secondary', 'test[' . $testCase . ']', false);
			}
			
			?>
		</form>

		<?php elseif($error = Pronamic_WordPress_IDeal_IDeal::getError()): ?>

		<div class="error inline below-h2">
			<dl>
				<dt><?php _e('Code', 'pronamic_ideal'); ?></dt>
				<dd><?php echo $error->getCode(); ?></dd>

				<dt><?php _e('Message', 'pronamic_ideal'); ?></dt>
				<dd><?php echo $error->getMessage(); ?></dd>

				<dt><?php _e('Detail', 'pronamic_ideal'); ?></dt>
				<dd><?php echo $error->getDetail(); ?></dd>

				<dt><?php _e('Consumer Message', 'pronamic_ideal'); ?></dt>
				<dd><?php echo $error->getConsumerMessage(); ?></dd>
			</dl>
		</div>

		<?php endif; ?>
	</div>
	
	<?php endif; ?>

	<?php if($configuration->getVariant() instanceof Pronamic_IDeal_VariantBasic): ?>
	
	<h3>
		<?php _e('Mandatory Tests', 'pronamic_ideal'); ?>
	</h3>

	<?php foreach(array(1, 2, 3, 4, 5, 6, 7) as $testCase): ?>
	
	<?php 
				
	$name = sprintf(__('Test Case %s', 'pronamic_ideal'), $testCase);
			
	$iDeal = new Pronamic_IDeal_Basic();
	$iDeal->setPaymentServerUrl($configuration->getPaymentServerUrl());
	$iDeal->setMerchantId($configuration->getMerchantId());
	$iDeal->setSubId($configuration->getSubId());
	$iDeal->setLanguage(Pronamic_WordPress_IDeal_Util::getLanguageIso639Code());
	$iDeal->setHashKey($configuration->hashKey);
	$iDeal->setCurrency('EUR');
	$iDeal->setPurchaseId(uniqid('test-' . $testCase));
	$iDeal->setDescription('Test ' . $testCase);

	// Success URL
	$url = add_query_arg('status', 'success', $testsLink);
	$iDeal->setSuccessUrl($url);

	// Cancel URL
	$url = add_query_arg('status', 'cancel', $testsLink);
	$iDeal->setCancelUrl($url);

	// Error URL
	$url = add_query_arg('status', 'error', $testsLink);
	$iDeal->setErrorUrl($url);

	// Test item
	$items = new Pronamic_IDeal_Items();

	$item = new Pronamic_IDeal_Item();
	$item->setNumber($testCase);
	$item->setDescription($name);
	$item->setPrice($testCase);
	$item->setQuantity(1);
	
	$items->addItem($item);

	$iDeal->setItems($items);
	
	?>
	<form method="post" action="<?php echo esc_attr($iDeal->getPaymentServerUrl()); ?>" target="_blank" style="display: inline">
		<?php 
		
		echo $iDeal->getHtmlFields(); 

		submit_button($name, 'secondary', 'submit', false); 
						
		?>
	</form>

	<?php endforeach; ?>

	<?php endif; ?>

	<?php if($configuration->getVariant() instanceof Pronamic_IDeal_VariantKassa): ?>
	
	<?php 
	
	$user = wp_get_current_user();

	$iDeal = new Pronamic_Gateways_IDealInternetKassa_IDealInternetKassa();

	$iDeal->setPaymentServerUrl($configuration->getPaymentServerUrl());

	$iDeal->setPspId($configuration->pspId);
	$iDeal->setPassPhraseIn($configuration->shaInPassPhrase);
	$iDeal->setPassPhraseOut($configuration->shaOutPassPhrase);

	$iDeal->setOrderId(time());
	$iDeal->setAmount(1);
	$iDeal->setCurrency('EUR');
	$iDeal->setLanguage('nl_NL');
	$iDeal->setCustomerName($user->user_firstname . ' ' . $user->user_lastname);
	$iDeal->setEMailAddress($user->user_email);
	
	$iDeal->setField('PARAMPLUS', 'pid=1234567890');

	$file = dirname(Pronamic_WordPress_IDeal_Plugin::$file) . '/other/calculations-parameters-sha-in.txt';
	$iDeal->setCalculationsParametersIn( file($file, FILE_IGNORE_NEW_LINES) );

	$file = dirname(Pronamic_WordPress_IDeal_Plugin::$file) . '/other/calculations-parameters-sha-out.txt';
	$iDeal->setCalculationsParametersOut( file($file, FILE_IGNORE_NEW_LINES) );

	?>
	
	<h3>
		<?php _e('Mandatory Tests', 'pronamic_ideal'); ?>
	</h3>

	<form method="post" action="<?php echo esc_attr($iDeal->getPaymentServerUrl()); ?>" target="_blank" style="display: inline">
		<?php 
		
		echo $iDeal->getHtmlFields(); 

		submit_button('Test', 'secondary', 'submit', false); 
						
		?>
	</form>

	<?php endif; ?>

	<?php if($configuration->getVariant() instanceof Pronamic_IDeal_VariantOmniKassa): ?>
	
	<h3>
		<?php _e('Tests', 'pronamic_ideal'); ?>
	</h3>

	<?php foreach(array(2, 3, 4, 5, 1) as $testCase): ?>

	<?php 
				
	$name = sprintf(__('Test &euro; %s', 'pronamic_ideal'), $testCase);

	$iDeal = new Pronamic_IDeal_OmniKassa();

	$iDeal->setPaymentServerUrl($configuration->getPaymentServerUrl());
	$iDeal->setMerchantId($configuration->getMerchantId());
	$iDeal->setKeyVersion($configuration->getSubId());
	$iDeal->setSecretKey($configuration->hashKey);
	$iDeal->setCurrencyNumericCode(978);
	$iDeal->setNormalReturnUrl(site_url('/'));
	$iDeal->setAmount($testCase);
	$iDeal->setTransactionReference(uniqid('test'));
	// $iDeal->setOrderId(1);
	$iDeal->setCustomerLanguage(Pronamic_WordPress_IDeal_Util::getLanguageIso639Code());

	?>

	<form method="post" action="<?php echo esc_attr($iDeal->getPaymentServerUrl()); ?>" target="_blank" style="display: inline">
		<?php 

		echo $iDeal->getHtmlFields();

		submit_button($name, 'secondary', 'submit', false); 

		?>
	</form>

	<?php endforeach; ?>

	<?php endif; ?>

	<?php if($configuration->getVariant() instanceof Pronamic_IDeal_VariantEasy): ?>
	
	<h3>
		<?php _e('Tests', 'pronamic_ideal'); ?>
	</h3>

	<?php foreach(array(2, 3, 4, 5, 1) as $testCase): ?>

	<?php 
				
	$name = sprintf(__('Test &euro; %s', 'pronamic_ideal'), $testCase);
	
	$user = wp_get_current_user();

	$iDeal = new Pronamic_Gateways_IDealEasy_IDealEasy();

	$iDeal->setPaymentServerUrl($configuration->getPaymentServerUrl());
	$iDeal->setPspId($configuration->pspId);

	$iDeal->setLanguage(Pronamic_WordPress_IDeal_Util::getLanguageIso639AndCountryIso3166Code());
	$iDeal->setCurrency('EUR');
	$iDeal->setOrderId(uniqid('test'));
	$iDeal->setDescription($name);
	$iDeal->setAmount($testCase);
	$iDeal->setEMailAddress($user->user_email);
	$iDeal->setCustomerName($user->user_firstname . ' ' . $user->user_lastname);

	?>

	<form method="post" action="<?php echo esc_attr($iDeal->getPaymentServerUrl()); ?>" target="_blank" style="display: inline">
		<?php 

		echo $iDeal->getHtmlFields();

		submit_button($name, 'secondary', 'submit', false); 

		?>
	</form>

	<?php endforeach; ?>

	<?php endif; ?>

	<?php endif; ?>
</div>